//  Control a Servo using the Interrupt Capability available in the "TimerOne" Library
//
//  Potentiometer used to set servo position
//
//  Written by myke predko

#include <TimerOne.h>

//  Set Servo outputs using Timer Interrupts

int pot1pin = 3;   // analog pin used to connect the potentiometer
int state   = 2;   //  0 - Start Servo Pulse
                   //  1 - Wait for Pulse to End
                   //  2 - Delay to Next Pulse
int servo1value;  
int intservo1value;  
int val;

void setup() {
  
    pinMode(9, OUTPUT);
  
    state = 2;
    Timer1.initialize(100000); // set a timer of length 100000 microseconds (or 0.1 sec - or 10Hz => the led will blink 5 times, 5 cycles of on-and-off, per second)
    Timer1.attachInterrupt(timerIsr); // attach the service routine here
    
}
 
void loop() {

    val         = analogRead(pot1pin);            // reads the value of the potentiometer (value between 0 and 1023) 
    servo1value = map(val, 0, 1023, 600, 2400);     // scale it to use it with the servo (value between 600 and 2400ms) 

    delay(10);  //  Wait 10ms for next sample

}
 
/// --------------------------
/// Custom ISR Timer Routine
/// --------------------------
void timerIsr()
{
    switch(state) {
    case 0:  //  Pulse High
      digitalWrite(9, HIGH);
      Timer1.setPeriod(intservo1value);
      state = 1;
      break;
    case 1:  //  Pulse Low and delay to 2400us
      digitalWrite(9, LOW);
      Timer1.setPeriod(2500 - intservo1value);
      state = 2;
      break;
    case 2:  //  Wait to restart at 20ms
      intservo1value = servo1value;
      Timer1.setPeriod(17500);
      state = 0;
      break;
    }
}
