//  Control two Servos using the Interrupt Capability available in the "TimerOne" Library
//
//  Potentiometer used to set servo position
//
//  Written by myke predko

#include <TimerOne.h>

//  Set Servo outputs using Timer Interrupts

int pot1pin = 3;   // analog pin used to connect the potentiometer
int pot2pin = 2;  
int state   = 4;   //  0 - Start Servo1 Pulse
                   //  1 - Wait for Servo1 Pulse to End
                   //  2 - Start Servo2 Pulse
                   //  3 - Wait for Servo2 Pulse to End
                   //  4 - Delay to Next Pulse (15ms)
int servo1value;  
int intservo1value;  
int servo2value;  
int intservo2value;  
int val;

void setup() {
  
    pinMode(9, OUTPUT);
    pinMode(10, OUTPUT);
  
    state = 4;
    Timer1.initialize(100000); // set a timer of length 100000 microseconds (or 0.1 sec - or 10Hz => the led will blink 5 times, 5 cycles of on-and-off, per second)
    Timer1.attachInterrupt(timerIsr); // attach the service routine here
    
}
 
void loop() {

    val         = analogRead(pot1pin);            // reads the value of the potentiometer (value between 0 and 1023) 
    servo1value = map(val, 0, 1023, 600, 2400);     // scale it to use it with the servo (value between 600 and 2400ms) 

    val         = analogRead(pot2pin);            
    servo2value = map(val, 0, 1023, 600, 2400);   

    delay(10);  //  Wait 10ms for next sample

}
 
/// --------------------------
/// Custom ISR Timer Routine
/// --------------------------
void timerIsr()
{
    switch(state) {
    case 0:  //  Pulse High
      digitalWrite(9, HIGH);
      Timer1.setPeriod(intservo1value);
      state = 1;
      break;
    case 1:  //  Pulse Low and delay to 2400us
      digitalWrite(9, LOW);
      Timer1.setPeriod(2500 - intservo1value);
      state = 2;
      break;
    case 2:  //  Pulse High
      digitalWrite(10, HIGH);
      Timer1.setPeriod(intservo2value);
      state = 3;
      break;
    case 3:  //  Pulse Low and delay to 2400us
      digitalWrite(10, LOW);
      Timer1.setPeriod(2500 - intservo2value);
      state = 4;
      break;
    case 4:  //  Wait to restart at 20ms
      intservo1value = servo1value;
      intservo2value = servo2value;
      Timer1.setPeriod(15000);
      state = 0;
      break;
    }
}
